const readline = require( "readline" );

function checkInterfaces( state ) {
  _.each( state.streams, ( stream ) => {
    if( !stream.reader ) {
      let reader = readline.createInterface( {
        input: stream.stream
      } );
      stream.reader = reader;
      reader.on( "line", ( raw ) => {
        stream.linesRead ++;
        let line = JSON.parse( raw );
        line._filename = stream.file;
        state.stacks.default.execute( {}, line )
          .then( () => updatePosition( state, stream ) );
      } );
    }
  } );
  setTimer( state );
}

function refresh( state ) {
  return state.logs.getAllStreams( state.streams )
    .then( ( streams ) => {
      state.streams = streams;
      return checkInterfaces( state );
    } );
}

function setTimer( state ) {
  state.pendingTimeout = setTimeout( function() {
    refresh( state );
  }, state.config.scanFrequency );
}

function start( state ) {
  return refresh( state );
}

function stop( state ) {
  if( state.pendingTimeout ) {
    clearTimeout( state.pendingTimeout );
  }
}

function updatePosition( state, stream ) {
  stream.start = stream.linesRead > stream.start ? stream.linesRead : stream.start;
  state.entries[ stream.file ] = stream.linesRead;
  state.logs.storeLastEntries( state.entries );
}

module.exports = function( state ) {
  return {
    checkInterfaces: checkInterfaces.bind( null, state ),
    refresh: refresh.bind( null, state ),
    setTimer: setTimer.bind( null, state ),
    start: start.bind( null, state ),
    stop: stop.bind( null, state ),
    updatePosition: updatePosition.bind( null, state )
  }
}